@@ -887,7 +887,7 @@ Wrinkles:
 
   Plainly the (C a) constraint is unused; but the expanded decl will look like
         $dmop2 :: C a => a -> a
-        $dmop2 = op1 . op2
+        $dmop2 = op1 . op1
 
         $fCList :: forall a. C a => C [a]
         $fCList @a (d::C a) = MkC (\(x:a).x) ($dmop2 @a d)
@@ -902,10 +902,12 @@ Wrinkles:
    It's a bit of a palaver, but not really difficult.
    All the logic is localised in `neededEvVars`.
 
-
-
------ Reporting redundant constraints
-
+   But NOTE that this only applies to /vanilla/ default methods.
+   For /generic/ default methods, like
+            class D a where { op1 :: blah
+                            ; default op1 :: Eq a => blah2 }
+   the (Eq a) constraint really is needed (e.g. class NFData and #25992).
+   Hence the `Bool` field of `MethSkol` indicates a /vanilla/ default method.
 
 ----- Examples
 
@@ -1679,8 +1681,9 @@ runTcPluginsWanted wanted
        ; if null solvers then return (False, wanted) else
 
     do { -- Find the set of Givens to give to the plugin.
-         -- If TcSMode = TcSShortCut, we are solving with
-         -- no Givens so don't return any (#26258)!
+         -- Optimisation: if TcSMode = TcSShortCut, we are solving with
+         -- no Givens so don't bother to look (#26258 was a bug in an earlier
+         -- version when we left the Givens in the inert set)
          -- See Note [Shortcut solving] in GHC.Tc.Solver.Dict
          mode <- getTcSMode
        ; given <- case mode of
