@@ -156,6 +156,7 @@ stage1Packages = do
         , ghcInternal
         , ghcPkg
         , ghcPrim
+        , haddock
         , haskeline
         , hp2ps
         , hsc2hs
@@ -174,8 +175,7 @@ stage1Packages = do
         , if winTarget then win32 else unix
         ]
       , when (not cross)
-        [ haddock
-        , hpcBin
+        [ hpcBin
         , iserv
         , runGhc
         , ghcToolchainBin
@@ -277,13 +277,14 @@ defaultFlavour = Flavour
     , libraryWays        = defaultLibraryWays
     , rtsWays            = defaultRtsWays
     , dynamicGhcPrograms = defaultDynamicGhcPrograms
-    , ghciWithDebugger   = const False
     , ghcProfiled        = const False
     , ghcDebugged        = const False
     , ghcThreaded        = const True
     , ghcDebugAssertions = const False
     , ghcSplitSections   = False
-    , ghcDocs            = cmdDocsArgs }
+    , ghcDocs            = cmdDocsArgs
+    , ghcHieFiles        = const False
+    , hashUnitIds        = False }
 
 -- | Default logic for determining whether to build
 --   dynamic GHC programs.
