@@ -4,7 +4,6 @@ module Main (main) where
 import Control.Lens     (imap)
 import Data.Aeson       (FromJSON (..), eitherDecode, withObject, (.:))
 import Data.List        (sortOn)
-import Data.Semigroup   ((<>))
 import Data.Text        (Text)
 import Data.Traversable (for)
 
@@ -32,6 +31,12 @@ main = generate =<< O.execParser opts where
         <$> licenses "3.0"
         <*> licenses "3.2"
         <*> licenses "3.6"
+        <*> licenses "3.9"
+        <*> licenses "3.10"
+        <*> licenses "3.16"
+        <*> licenses "3.23"
+        <*> licenses "3.25"
+        <*> licenses "3.26"
 
     template = O.strArgument $ mconcat
         [ O.metavar "SPDX.LicenseExceptionId.template.hs"
@@ -61,26 +66,17 @@ generate'
     -> (Input -> IO String)
     -> IO String
 generate' lss template = template $ Input
-    { inputLicenseIds      = licenseIds
-    , inputLicenses        = licenseValues
-    , inputLicenseList_all = mkLicenseList (== allVers)
-    , inputLicenseList_3_0 = mkLicenseList
-        (\vers -> vers /= allVers && Set.member SPDXLicenseListVersion_3_0 vers)
-    , inputLicenseList_3_2 = mkLicenseList
-        (\vers -> vers /= allVers && Set.member SPDXLicenseListVersion_3_2 vers)
-    , inputLicenseList_3_6 = mkLicenseList
-        (\vers -> vers /= allVers && Set.member SPDXLicenseListVersion_3_6 vers)
+    { inputLicenseIds       = licenseIds
+    , inputLicenses         = licenseValues
+    , inputLicenseList_all  = mkLicenseList (== allVers)
+    , inputLicenseList_perv = tabulate $ \ver -> mkLicenseList
+        (\vers -> vers /= allVers && Set.member ver vers)
     }
   where
-    PerV (LL ls_3_0) (LL ls_3_2) (LL ls_3_6) = lss
-
     constructorNames :: [(Text, License, Set.Set SPDXLicenseListVersion)]
     constructorNames
         = map (\(l, tags) -> (toConstructorName $ licenseId l, l, tags))
-        $ combine licenseId $ \ver -> case ver of
-            SPDXLicenseListVersion_3_6 -> filterDeprecated ls_3_6
-            SPDXLicenseListVersion_3_2 -> filterDeprecated ls_3_2
-            SPDXLicenseListVersion_3_0 -> filterDeprecated ls_3_0
+        $ combine licenseId $ \ver -> filterDeprecated $ unLL $ index ver lss
 
     filterDeprecated = filter (not . licenseDeprecated)
 
@@ -90,6 +86,7 @@ generate' lss template = template $ Input
         , ilId            = textShow (licenseId l)
         , ilName          = textShow (licenseName l)
         , ilIsOsiApproved = False -- not used in exceptions
+        , ilIsFsfLibre    = False -- not used in exceptions
         }
 
     licenseIds :: Text
@@ -123,7 +120,7 @@ instance FromJSON License where
         fixSpace '\n' = ' '
         fixSpace c =   c
 
-newtype LicenseList = LL [License]
+newtype LicenseList = LL { unLL :: [License] }
   deriving (Show)
 
 instance FromJSON LicenseList where
