@@ -1,4 +1,5 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE NondecreasingIndentation #-}
 -- | A GHC run-server, which supports running multiple GHC scripts
@@ -40,6 +41,9 @@ import qualified GHC.IO.Exception as GHC
 import Distribution.Verbosity
 
 import System.Process.Internals
+  ( ProcessHandle__( OpenHandle )
+  , withProcessHandle
+  )
 #if mingw32_HOST_OS
 import qualified System.Win32.Process as Win32
 #endif
@@ -136,7 +140,7 @@ runOnServer :: Server -> Maybe FilePath -> [(String, Maybe String)]
             -> FilePath -> [String] -> IO ServerResult
 runOnServer s mb_cwd env_overrides script_path args = do
     -- TODO: cwd not implemented
-    when (isJust mb_cwd)            $ error "runOnServer change directory not implemented"
+    when (isJust mb_cwd)        $ error "runOnServer change directory not implemented"
     -- TODO: env_overrides not implemented
     unless (null env_overrides) $ error "runOnServer set environment not implemented"
 
@@ -181,12 +185,13 @@ runOnServer s mb_cwd env_overrides script_path args = do
     -- Give the user some indication about how they could run the
     -- command by hand.
     (real_path, real_args) <- runnerCommand (serverScriptEnv s) mb_cwd env_overrides script_path args
-    return ServerResult {
-            serverResultTestCode = code,
-            serverResultCommand = showCommandForUser real_path real_args,
-            serverResultStdout = out,
-            serverResultStderr = err
-        }
+    return $
+      ServerResult {
+              serverResultTestCode = code,
+              serverResultCommand = showCommandForUser real_path real_args,
+              serverResultStdout = out,
+              serverResultStderr = err
+          }
 
 -- | Helper function which we use in the GHCi session to communicate
 -- the exit code of the process.
@@ -215,7 +220,7 @@ runMain ref m = do
 startServer :: Chan ServerLogMsg -> ScriptEnv -> IO Server
 startServer chan senv = do
     (prog, _) <- requireProgram verbosity ghcProgram (runnerProgramDb senv)
-    let ghc_args = runnerGhcArgs senv ++ ["--interactive", "-v0", "-ignore-dot-ghci"]
+    let ghc_args = runnerGhcArgs senv Nothing ++ ["--interactive", "-v0", "-ignore-dot-ghci"]
         proc_spec = (proc (programPath prog) ghc_args) {
                         create_group = True,
                         -- Closing fds is VERY important to avoid
@@ -226,7 +231,6 @@ startServer chan senv = do
                         std_out = CreatePipe,
                         std_err = CreatePipe
                     }
-    -- printRawCommandAndArgsAndEnv (runnerVerbosity senv) (programPath prog) ghc_args Nothing
     when (verbosity >= verbose) $
         writeChan chan (ServerLogMsg AllServers (showCommandForUser (programPath prog) ghc_args))
     (Just hin, Just hout, Just herr, proch) <- createProcess proc_spec
@@ -249,26 +253,20 @@ startServer chan senv = do
 
 -- | Unmasked initialization for the server
 initServer :: Server -> IO Server
+{- FOURMOLU_DISABLE -}
 initServer s0 = do
     -- NB: withProcessHandle reads an MVar and is interruptible
+
+    pid <- withProcessHandle (serverProcessHandle s0) $ \ph ->
+              case ph of
 #if mingw32_HOST_OS
-    pid <- withProcessHandle (serverProcessHandle s0) $ \ph ->
-              case ph of
                   OpenHandle x   -> fmap show (Win32.getProcessId x)
-                  ClosedHandle  _ -> return (serverProcessId s0)
 #else
-    pid <- withProcessHandle (serverProcessHandle s0) $ \ph ->
-              case ph of
-#if MIN_VERSION_process(1,2,0)
                   OpenHandle x   -> return (show x)
+#endif
                   -- TODO: handle OpenExtHandle?
                   _              -> return (serverProcessId s0)
-#else
-                  OpenHandle x   -> return (ph, show x)
-                  -- TODO: handle OpenExtHandle?
-                  _              -> return (ph, serverProcessId s0)
-#endif
-#endif
+
     let s = s0 { serverProcessId = pid }
     -- We will read/write a line at a time, including for
     -- output; our demarcation tokens are an entire line.
@@ -278,6 +276,7 @@ initServer s0 = do
     write s ":set prompt \"\""
     write s "System.IO.hSetBuffering System.IO.stdout System.IO.LineBuffering"
     return s
+{- FOURMOLU_ENABLE -}
 
 -- | Stop a GHCi session.
 stopServer :: Server -> IO ()
