@@ -418,10 +418,7 @@ data EvalStatus_ a b
 instance Binary a => Binary (EvalStatus_ a b)
 
 data EvalBreakpoint = EvalBreakpoint
-  { eb_tick_mod      :: String -- ^ Breakpoint tick module
-  , eb_tick_mod_unit :: BS.ShortByteString -- ^ Breakpoint tick module unit id
-  , eb_tick_index    :: Int    -- ^ Breakpoint tick index
-  , eb_info_mod      :: String -- ^ Breakpoint info module
+  { eb_info_mod      :: String -- ^ Breakpoint info module
   , eb_info_mod_unit :: BS.ShortByteString -- ^ Breakpoint tick module unit id
   , eb_info_index    :: Int    -- ^ Breakpoint info index
   }
