@@ -81,7 +81,7 @@ hashWord(const HashTable *table, StgWord key)
     int bucket;
 
     /* Strip the boring zero bits */
-    key >>= sizeof(StgWord);
+    key /= sizeof(StgWord);
 
     /* Mod the size of the hash table (a power of 2) */
     bucket = key & table->mask1;
@@ -94,13 +94,13 @@ hashWord(const HashTable *table, StgWord key)
 }
 
 int
-hashStr(const HashTable *table, StgWord w)
+hashBuffer(const HashTable *table, const void *buf, size_t len)
 {
-    const char *key = (char*) w;
+    const char *key = (char*) buf;
 #if WORD_SIZE_IN_BITS == 64
-    StgWord h = XXH3_64bits_withSeed (key, strlen(key), 1048583);
+    StgWord h = XXH3_64bits_withSeed (key, len, 1048583);
 #else
-    StgWord h = XXH32 (key, strlen(key), 1048583);
+    StgWord h = XXH32 (key, len, 1048583);
 #endif
 
     /* Mod the size of the hash table (a power of 2) */
@@ -114,6 +114,13 @@ hashStr(const HashTable *table, StgWord w)
     return bucket;
 }
 
+int
+hashStr(const HashTable *table, StgWord w)
+{
+    const char *key = (char*) w;
+    return hashBuffer(table, key, strlen(key));
+}
+
 STATIC_INLINE int
 compareWord(StgWord key1, StgWord key2)
 {
