@@ -2,7 +2,7 @@
  * (c)2006 Galois Connections, Inc.
  */
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 
 #include "Trace.h"
@@ -45,7 +45,7 @@ HpcModuleInfo *modules = 0;
 
 static char *tixFilename = NULL;
 
-static void GNU_ATTRIBUTE(__noreturn__)
+static void STG_NORETURN
 failure(char *msg) {
   debugTrace(DEBUG_hpc,"hpc failure: %s\n",msg);
   fprintf(stderr,"Hpc failure: %s\n",msg);
@@ -137,7 +137,7 @@ readTix(void) {
     tmpModule -> hashNo = (unsigned int)expectWord64();
     ws();
     tmpModule -> tickCount = (int)expectWord64();
-    tmpModule -> tixArr = (StgWord64 *)calloc(tmpModule->tickCount,sizeof(StgWord64));
+    tmpModule -> tixArr = (StgWord64 *)stgCallocBytes(tmpModule->tickCount,sizeof(StgWord64), "readTix");
     ws();
     expect('[');
     ws();
@@ -203,7 +203,9 @@ startupHpc(void)
     return;
   }
   hpc_inited = 1;
+#if defined(HAVE_GETPID)
   hpc_pid    = getpid();
+#endif
   hpc_tixdir = getenv("HPCTIXDIR");
   hpc_tixfile = getenv("HPCTIXFILE");
 
@@ -234,7 +236,14 @@ startupHpc(void)
     sprintf(tixFilename, "%s.tix", prog_name);
   }
 
-  if (init_open(__rts_fopen(tixFilename,"r"))) {
+  if ((RtsFlags.HpcFlags.readTixFile == HPC_YES_IMPLICIT) && init_open(__rts_fopen(tixFilename,"r"))) {
+    fprintf(stderr,"Deprecation warning:\n"
+                   "I am reading in the existing tix file, and will add hpc info from this run to the existing data in that file.\n"
+                   "GHC 9.14 will cease looking for an existing tix file by default.\n"
+                   "If you positively want to add hpc info to the current tix file, use the RTS option --read-tix-file=yes.\n"
+                   "More information can be found in the accepted GHC proposal 612.\n");
+    readTix();
+  } else if ((RtsFlags.HpcFlags.readTixFile == HPC_YES_EXPLICIT) && init_open(__rts_fopen(tixFilename,"r"))) {
     readTix();
   }
 }
@@ -387,7 +396,12 @@ exitHpc(void) {
   // Any sub-process from use of fork from inside Haskell will
   // not clobber the .tix file.
 
-  if (hpc_pid == getpid()) {
+#if defined(HAVE_GETPID)
+  bool is_subprocess = hpc_pid != getpid();
+#else
+  bool is_subprocess = false;
+#endif
+  if (!is_subprocess && RtsFlags.HpcFlags.writeTixFile) {
     FILE *f = __rts_fopen(tixFilename,"w+");
     writeTix(f);
   }
