@@ -51,7 +51,7 @@
 /* often times we need to extend some value of certain number of bits
  * int an int64_t for e.g. relative offsets.
  */
-int64_t signExtend(uint64_t val, uint8_t bits);
+static int64_t signExtend(uint64_t val, uint8_t bits);
 /* Helper functions to check some instruction properties */
 static bool isVectorOp(uint32_t *p);
 static bool isLoadStore(uint32_t *p);
@@ -60,17 +60,18 @@ static bool isLoadStore(uint32_t *p);
  * where we want to write the address offset to. Thus decoding as well
  * as encoding is needed.
  */
-bool fitsBits(size_t bits, int64_t value);
-int64_t decodeAddend(ObjectCode * oc, Section * section,
+static bool fitsBits(size_t bits, int64_t value);
+static int64_t decodeAddend(ObjectCode * oc, Section * section,
                      MachORelocationInfo * ri);
-void encodeAddend(ObjectCode * oc, Section * section,
+static void encodeAddend(ObjectCode * oc, Section * section,
                   MachORelocationInfo * ri, int64_t addend);
 
 /* Global Offset Table logic */
-bool isGotLoad(MachORelocationInfo * ri);
-bool needGotSlot(MachONList * symbol);
-bool makeGot(ObjectCode * oc);
-void freeGot(ObjectCode * oc);
+static bool isGotLoad(MachORelocationInfo * ri);
+static bool needGotSlot(MachOSymbol * symbol);
+static bool makeGot(ObjectCode * oc);
+static void freeGot(ObjectCode * oc);
+static void findInternalGotRefs(ObjectCode * oc);
 #endif /* aarch64_HOST_ARCH */
 
 /*
@@ -252,7 +253,7 @@ resolveImports(
             return 0;
         }
 
-        checkProddableBlock(oc,
+        checkProddableBlock(&oc->proddables,
                             ((void**)(oc->image + sect->offset)) + i,
                             sizeof(void *));
         ((void**)(oc->image + sect->offset))[i] = addr;
@@ -265,7 +266,7 @@ resolveImports(
 #if defined(aarch64_HOST_ARCH)
 /* aarch64 linker by moritz angermann <moritz@lichtzwerge.de> */
 
-int64_t
+static int64_t
 signExtend(uint64_t val, uint8_t bits) {
     return (int64_t)(val << (64-bits)) >> (64-bits);
 }
@@ -280,22 +281,32 @@ isLoadStore(uint32_t *p) {
     return (*p & 0x3B000000) == 0x39000000;
 }
 
-int64_t
+static int64_t
 decodeAddend(ObjectCode * oc, Section * section, MachORelocationInfo * ri) {
 
     /* the instruction. It is 32bit wide */
     uint32_t * p = (uint32_t*)((uint8_t*)section->start + ri->r_address);
 
-    checkProddableBlock(oc, (void*)p, 1 << ri->r_length);
+    checkProddableBlock(&oc->proddables, (void*)p, 1 << ri->r_length);
 
     switch(ri->r_type) {
-        case ARM64_RELOC_UNSIGNED:
+        case ARM64_RELOC_UNSIGNED: {
+            switch (ri->r_length) {
+                case 0: return signExtend(*(uint8_t*)p,  8 << ri->r_length);
+                case 1: return signExtend(*(uint16_t*)p, 8 << ri->r_length);
+                case 2: return signExtend(*(uint32_t*)p, 8 << ri->r_length);
+                case 3: return signExtend(*(uint64_t*)p, 8 << ri->r_length);
+                default:
+                    barf("Unsupported r_length (%d) for UNSIGNED relocation",
+                         ri->r_length);
+            }
+        }
         case ARM64_RELOC_SUBTRACTOR: {
             switch (ri->r_length) {
-                case 0: return signExtend(*(uint8_t*)p,  8 * (1 << ri->r_length));
-                case 1: return signExtend(*(uint16_t*)p, 8 * (1 << ri->r_length));
-                case 2: return signExtend(*(uint32_t*)p, 8 * (1 << ri->r_length));
-                case 3: return signExtend(*(uint64_t*)p, 8 * (1 << ri->r_length));
+                case 0: return signExtend(*(uint8_t*)p,  8 << ri->r_length);
+                case 1: return signExtend(*(uint16_t*)p, 8 << ri->r_length);
+                case 2: return signExtend(*(uint32_t*)p, 8 << ri->r_length);
+                case 3: return signExtend(*(uint64_t*)p, 8 << ri->r_length);
                 default:
                     barf("Unsupported r_length (%d) for SUBTRACTOR relocation",
                          ri->r_length);
@@ -340,7 +351,7 @@ decodeAddend(ObjectCode * oc, Section * section, MachORelocationInfo * ri) {
     barf("unsupported relocation type: %d\n", ri->r_type);
 }
 
-inline bool
+inline static bool
 fitsBits(size_t bits, int64_t value) {
     if(bits == 64) return true;
     if(bits > 64) barf("fits_bits with %zu bits and an 64bit integer!", bits);
@@ -348,18 +359,31 @@ fitsBits(size_t bits, int64_t value) {
         || -1 == (value >> bits);  // All bits on: -1
 }
 
-void
+static void
 encodeAddend(ObjectCode * oc, Section * section,
              MachORelocationInfo * ri, int64_t addend) {
     uint32_t * p = (uint32_t*)((uint8_t*)section->start + ri->r_address);
 
-    checkProddableBlock(oc, (void*)p, 1 << ri->r_length);
+    checkProddableBlock(&oc->proddables, (void*)p, 1 << ri->r_length);
 
     switch (ri->r_type) {
-        case ARM64_RELOC_UNSIGNED:
+        case ARM64_RELOC_UNSIGNED: {
+            if(!fitsBits(8 << ri->r_length, addend))
+                barf("Relocation out of range for UNSIGNED");
+            switch (ri->r_length) {
+                case 0: *(uint8_t*)p  = (uint8_t)addend; break;
+                case 1: *(uint16_t*)p = (uint16_t)addend; break;
+                case 2: *(uint32_t*)p = (uint32_t)addend; break;
+                case 3: *(uint64_t*)p = (uint64_t)addend; break;
+                default:
+                    barf("Unsupported r_length (%d) for UNSIGNED relocation",
+                         ri->r_length);
+            }
+            return;
+        }
         case ARM64_RELOC_SUBTRACTOR: {
             if(!fitsBits(8 << ri->r_length, addend))
-                barf("Relocation out of range for UNSIGNED/SUBTRACTOR");
+                barf("Relocation out of range for SUBTRACTOR");
             switch (ri->r_length) {
                 case 0: *(uint8_t*)p  = (uint8_t)addend; break;
                 case 1: *(uint16_t*)p = (uint16_t)addend; break;
@@ -417,7 +441,49 @@ encodeAddend(ObjectCode * oc, Section * section,
     barf("unsupported relocation type: %d\n", ri->r_type);
 }
 
-bool
+/* Note [Symbols in need of GOT entries]
+ * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+ * As GOT entries require memory, we ideally want to avoid reserving
+ * them for symbols where they are unnecessary. Specifically, most internal
+ * symbols will not be referenced by the GOT, even in position independent code
+ * (since you can instead use direct PC-relative addressing).
+ *
+ * However, it is nevertheless possible for internal symbols to be referenced
+ * via the GOT. Consequently, we use the following strategy to determine whether
+ * a symbol needs a GOT slot:
+ *
+ *  a. all undefined external symbols are given GOT entries
+ *  b. all external symbols with cross-section refrences are given GOT entries
+ *  c. all internal symbols for which there are GOT relocations are given GOT
+ *     entries.
+ *
+ * Failing to consider (c) lead to #25577. For this we explicitly traverse
+ * the relocations in findInternalGotRefs() looking for GOT relocations
+ * referencing internal symbols, setting the MachOSymbol.needs_got flag for
+ * each.
+ */
+
+// See Note [Symbols in need of GOT entries]
+static void
+findInternalGotRefs(ObjectCode * oc)
+{
+    for (int curSection = 0; curSection < oc->n_sections; curSection++) {
+        Section * sect = &oc->sections[curSection];
+        if (sect->info == NULL)
+            continue;
+        MachOSection * msect = sect->info->macho_section; // for access convenience
+        MachORelocationInfo * relocs = sect->info->relocation_info;
+        for(uint32_t i = 0; i < msect->nreloc; i++) {
+            MachORelocationInfo *ri = &relocs[i];
+            if (isGotLoad(ri)) {
+                MachOSymbol* symbol = &oc->info->macho_symbols[ri->r_symbolnum];
+                symbol->needs_got = true;
+            }
+        }
+    }
+}
+
+static bool
 isGotLoad(struct relocation_info * ri) {
     return ri->r_type == ARM64_RELOC_GOT_LOAD_PAGE21
     ||  ri->r_type == ARM64_RELOC_GOT_LOAD_PAGEOFF12;
@@ -425,22 +491,27 @@ isGotLoad(struct relocation_info * ri) {
 
 /*
  * Check if we need a global offset table slot for a
- * given symbol
+ * given symbol. See Note [Symbols in need of GOT entries].
  */
-bool
-needGotSlot(MachONList * symbol) {
-    return (symbol->n_type & N_EXT)             /* is an external symbol      */
-        && (N_UNDF == (symbol->n_type & N_TYPE) /* and is undefined           */
-            || NO_SECT != symbol->n_sect);      /*     or is defined in a
-                                                 *        different section   */
+static bool
+needGotSlot(MachOSymbol * symbol) {
+    // Does it have any internal references?
+    if (symbol->needs_got) {
+        return true;
+    }
+
+    return (symbol->nlist->n_type & N_EXT)             /* is an external symbol    */
+        && (N_UNDF == (symbol->nlist->n_type & N_TYPE) /* and is undefined         */
+            || NO_SECT != symbol->nlist->n_sect);      /*     or is defined in a
+                                                        *        different section */
 }
 
-bool
+static bool
 makeGot(ObjectCode * oc) {
     size_t got_slots = 0;
 
     for(size_t i=0; i < oc->info->n_macho_symbols; i++)
-        if(needGotSlot(oc->info->macho_symbols[i].nlist))
+        if(needGotSlot(&oc->info->macho_symbols[i]))
             got_slots += 1;
 
     if(got_slots > 0) {
@@ -453,7 +524,7 @@ makeGot(ObjectCode * oc) {
         /* update got_addr */
         size_t slot = 0;
         for(size_t i=0; i < oc->info->n_macho_symbols; i++)
-            if(needGotSlot(oc->info->macho_symbols[i].nlist))
+            if(needGotSlot(&oc->info->macho_symbols[i]))
                 oc->info->macho_symbols[i].got_addr
                     = ((uint8_t*)oc->info->got_start)
                     + (slot++ * sizeof(void *));
@@ -461,7 +532,7 @@ makeGot(ObjectCode * oc) {
     return EXIT_SUCCESS;
 }
 
-void
+static void
 freeGot(ObjectCode * oc) {
     /* sanity check */
     if(NULL != oc->info->got_start && oc->info->got_size > 0) {
@@ -471,6 +542,25 @@ freeGot(ObjectCode * oc) {
     oc->info->got_size = 0;
 }
 
+// Retrieve symbol value
+static uint64_t symbol_value(ObjectCode* oc, MachOSymbol* symbol) {
+  uint64_t value = 0;
+  if(symbol->nlist->n_type & N_EXT) {
+      /* external symbols should be able to be
+       * looked up via the lookupDependentSymbol function.
+       * Either through the global symbol hashmap
+       * or asking the system, if not found
+       * in the symbol hashmap
+       */
+      value = (uint64_t)lookupDependentSymbol((char*)symbol->name, oc, NULL);
+      if(!value)
+          barf("Could not lookup symbol: %s!", symbol->name);
+  } else {
+      value = (uint64_t)symbol->addr;    // address of the symbol.
+  }
+  return value;
+}
+
 static int
 relocateSectionAarch64(ObjectCode * oc, Section * section)
 {
@@ -498,43 +588,45 @@ relocateSectionAarch64(ObjectCode * oc, Section * section)
             case ARM64_RELOC_UNSIGNED: {
                 MachOSymbol* symbol = &oc->info->macho_symbols[ri->r_symbolnum];
                 int64_t addend = decodeAddend(oc, section, ri);
-                uint64_t value = 0;
-                if(symbol->nlist->n_type & N_EXT) {
-                    /* external symbols should be able to be
-                     * looked up via the lookupDependentSymbol function.
-                     * Either through the global symbol hashmap
-                     * or asking the system, if not found
-                     * in the symbol hashmap
-                     */
-                    value = (uint64_t)lookupDependentSymbol((char*)symbol->name, oc, NULL);
-                    if(!value)
-                        barf("Could not lookup symbol: %s!", symbol->name);
-                } else {
-                    value = (uint64_t)symbol->addr;    // address of the symbol.
-                }
+                uint64_t value = symbol_value(oc, symbol);
                 encodeAddend(oc, section, ri, value + addend);
                 break;
             }
             case ARM64_RELOC_SUBTRACTOR:
             {
-                MachOSymbol* symbol = &oc->info->macho_symbols[ri->r_symbolnum];
                 // subtractor and unsigned are called in tandem:
                 // first  pc <- pc - symbol address (SUBTRACTOR)
                 // second pc <- pc + symbol address (UNSIGNED)
                 // to achieve pc <- pc + target - base.
-                //
-                // the current implementation uses absolute addresses,
-                // which is simpler than trying to do this section
-                // relative, but could more easily lead to overflow.
-                //
+
+                // check that the following relocation exists and has the
+                // expected ARM64_RELOC_UNSIGNED type
                 if(!(i+1 < nreloc)
                    || !(section->info->relocation_info[i+1].r_type
                           == ARM64_RELOC_UNSIGNED))
                     barf("SUBTRACTOR relocation *must* be followed by UNSIGNED relocation.");
 
+                // we *know* that the next relocation is ARM64_RELOC_UNSIGNED
+                // (see above). So let's process both relocations and write the
+                // combined result in the target location. This prevents
+                // overflow. (Compared to trying to store the intermediate
+                // result which may not fit in the target bits).
+
+                // sub part (ARM64_RELOC_SUBTRACTOR)
+                MachOSymbol* symbol1 = &oc->info->macho_symbols[ri->r_symbolnum];
+                uint64_t sub_value = symbol_value(oc, symbol1);
+
+                // add part (ARM64_RELOC_UNSIGNED)
+                MachORelocationInfo * ri2 = &section->info->relocation_info[i+1];
+                MachOSymbol* symbol2 = &oc->info->macho_symbols[ri2->r_symbolnum];
+                uint64_t add_value = symbol_value(oc, symbol2);
+
+                // combine with addend and store
                 int64_t addend = decodeAddend(oc, section, ri);
-                int64_t value = (uint64_t)symbol->addr;
-                encodeAddend(oc, section, ri, addend - value);
+                encodeAddend(oc, section, ri, addend - sub_value + add_value);
+
+                // skip next relocation: we've already handled it
+                i += 1;
                 break;
             }
             case ARM64_RELOC_BRANCH26: {
@@ -583,6 +675,7 @@ relocateSectionAarch64(ObjectCode * oc, Section * section)
                     barf("explicit_addend and addend can't be set at the same time.");
                 uint64_t pc = (uint64_t)section->start + ri->r_address;
                 uint64_t value = (uint64_t)(isGotLoad(ri) ? symbol->got_addr : symbol->addr);
+                ASSERT(!isGotLoad(ri) || (symbol->got_addr != 0));
                 encodeAddend(oc, section, ri, ((value + addend + explicit_addend) & (-4096)) - (pc & (-4096)));
 
                 // reset, just in case.
@@ -596,6 +689,7 @@ relocateSectionAarch64(ObjectCode * oc, Section * section)
                 if(!(explicit_addend == 0 || addend == 0))
                     barf("explicit_addend and addend can't be set at the same time.");
                 uint64_t value = (uint64_t)(isGotLoad(ri) ? symbol->got_addr : symbol->addr);
+                ASSERT(!isGotLoad(ri) || (symbol->got_addr != 0));
                 encodeAddend(oc, section, ri, 0xFFF & (value + addend + explicit_addend));
 
                 // reset, just in case.
@@ -694,7 +788,7 @@ relocateSection(ObjectCode* oc, int curSection)
             default:
                 barf("Unknown size.");
         }
-        checkProddableBlock(oc,thingPtr,relocLenBytes);
+        checkProddableBlock(&oc->proddables,thingPtr,relocLenBytes);
 
         /*
          * With SIGNED_N the relocation is not at the end of the
@@ -940,9 +1034,9 @@ relocateSection(ObjectCode* oc, int curSection)
          */
         if (0 == reloc->r_extern) {
             if (reloc->r_pcrel) {
-                checkProddableBlock(oc, (void *)((char *)thing + baseValue), 1);
+                checkProddableBlock(&oc->proddables, (void *)((char *)thing + baseValue), 1);
             } else {
-                checkProddableBlock(oc, (void *)thing, 1);
+                checkProddableBlock(&oc->proddables, (void *)thing, 1);
             }
         }
 
@@ -1249,7 +1343,7 @@ ocGetNames_MachO(ObjectCode* oc)
                 secArray[sec_idx].info->stub_size = 0;
                 secArray[sec_idx].info->stubs = NULL;
 #endif
-                addProddableBlock(oc, start, section->size);
+                addProddableBlock(&oc->proddables, start, section->size);
             }
 
             curMem = (char*) secMem + section->size;
@@ -1406,6 +1500,8 @@ ocGetNames_MachO(ObjectCode* oc)
         }
     }
 #if defined(aarch64_HOST_ARCH)
+    findInternalGotRefs(oc);
+
     /* Setup the global offset table
      * This is for symbols that are external, and not defined here.
      * So that we can load their address indirectly.
@@ -1512,7 +1608,7 @@ ocResolve_MachO(ObjectCode* oc)
     /* fill the GOT table */
     for(size_t i = 0; i < oc->info->n_macho_symbols; i++) {
         MachOSymbol * symbol = &oc->info->macho_symbols[i];
-        if(needGotSlot(symbol->nlist)) {
+        if(needGotSlot(symbol)) {
             if(N_UNDF == (symbol->nlist->n_type & N_TYPE)) {
                 /* an undefined symbol. So we need to ensure we
                  * have the address.
