@@ -53,6 +53,19 @@ is_primtype :: Entry -> Bool
 is_primtype (PrimTypeSpec {}) = True
 is_primtype _ = False
 
+is_divLikeOp :: Entry -> Bool
+is_divLikeOp entry = case entry of
+   PrimOpSpec{} -> has_div_like
+   PseudoOpSpec{} -> has_div_like
+   PrimVecOpSpec{} -> has_div_like
+   PrimTypeSpec{} -> False
+   PrimVecTypeSpec{} -> False
+   Section{} -> False
+   where
+      has_div_like = case lookup_attrib "div_like" (opts entry) of
+         Just (OptionTrue{}) -> True
+         _ -> False
+
 -- a binding of property to value
 data Option
    = OptionFalse  String          -- name = False
@@ -63,6 +76,7 @@ data Option
    | OptionFixity (Maybe Fixity)  -- fixity = infix{,l,r} <int> | Nothing
    | OptionEffect PrimOpEffect    -- effect = NoEffect | DoNotSpeculate | CanFail | ThrowsException | ReadWriteEffect | FallibleReadWriteEffect
    | OptionCanFailWarnFlag PrimOpCanFailWarnFlag -- can_fail_warning = DoNotWarnCanFail | WarnIfEffectIsCanFail | YesWarnCanFail
+   | OptionDefinedBits (Maybe Word) -- defined_bits = Just 16 | Nothing
      deriving Show
 
 -- categorises primops
@@ -78,7 +92,7 @@ data Ty
    | TyVar  TyVar
    | TyUTup [Ty]   -- unboxed tuples; just a TyCon really,
                    -- but convenient like this
-   deriving (Eq,Show)
+   deriving (Eq,Show, Ord)
 
 type TyVar = String
 type TyVarBinder = String
@@ -183,6 +197,7 @@ get_attrib_name (OptionVector _) = "vector"
 get_attrib_name (OptionFixity _) = "fixity"
 get_attrib_name (OptionEffect _) = "effect"
 get_attrib_name (OptionCanFailWarnFlag _) = "can_fail_warning"
+get_attrib_name (OptionDefinedBits _) = "defined_bits"
 
 lookup_attrib :: String -> [Option] -> Maybe Option
 lookup_attrib _ [] = Nothing
